@@ -43,7 +43,9 @@ AutoCorrection::AutoCorrection()
       mReplaceSingleQuotes(false),
       mEnabled(false),
       mSuperScriptAppendix(false),
-      mAddNonBreakingSpace(false)
+      mAddNonBreakingSpace(false),
+      mMaxFindStringLenght(0),
+      mMinFindStringLenght(0)
 {
     // default double quote open 0x201c
     // default double quote close 0x201d
@@ -67,6 +69,44 @@ AutoCorrection::~AutoCorrection()
 {
 }
 
+void AutoCorrection::selectStringOnMaximumSearchString(QTextCursor &cursor, int cursorPosition)
+{
+    cursor.setPosition(cursorPosition);
+
+    QTextBlock block = cursor.block();
+    int pos = qMax(block.position(), cursorPosition - mMaxFindStringLenght);
+
+    //TODO verify that pos == block.position() => it's a full line => not a piece of word
+    //TODO if not => check if pos -1 is a space => not a piece of word
+    //TODO otherwise move cursor until we detect a space
+    //TODO otherwise we must not autoconvert it.
+    if (pos == block.position()) {
+        //it's ok
+    } else {
+        const QString text = block.text();
+        const int currentPos = (pos - block.position());
+        if (!text.at(currentPos - 1).isSpace()) {
+            //qDebug() << " current Text " << text << " currentPos "<< currentPos << " pos " << pos;
+            //qDebug() << "selected text " << text.right(text.length() - currentPos);
+            //qDebug() << "  text after " << text.at(currentPos - 1);
+            bool foundWord = false;
+            const int textLength(text.length());
+            for (int i = currentPos; i < textLength; ++i) {
+                if (text.at(i).isSpace()) {
+                    pos = qMin(cursorPosition, pos + 1 + block.position());
+                    foundWord = true;
+                    break;
+                }
+            }
+            if (!foundWord) {
+                pos = cursorPosition;
+            }
+        }
+    }
+    cursor.setPosition(pos);
+    cursor.setPosition(cursorPosition, QTextCursor::KeepAnchor);
+}
+
 void AutoCorrection::selectPreviousWord(QTextCursor &cursor, int cursorPosition)
 {
     cursor.setPosition(cursorPosition);
@@ -97,69 +137,71 @@ void AutoCorrection::selectPreviousWord(QTextCursor &cursor, int cursorPosition)
 
 bool AutoCorrection::autocorrect(bool htmlMode, QTextDocument &document, int &position)
 {
-    if (!mEnabled) {
-        return true;
-    }
-    mCursor =  QTextCursor(&document);
-    mCursor.setPosition(position);
+    if (mEnabled) {
+        mCursor =  QTextCursor(&document);
+        mCursor.setPosition(position);
 
-    //If we already have a space not necessary to look at other autocorrect feature.
-    if (!singleSpaces()) {
-        return false;
-    }
+        //If we already have a space not necessary to look at other autocorrect feature.
+        if (!singleSpaces()) {
+            return false;
+        }
 
-    int oldPosition = position;
-    selectPreviousWord(mCursor, position);
-    mWord = mCursor.selectedText();
-    if (mWord.isEmpty()) {
-        return true;
-    }
-    mCursor.beginEditBlock();
-    bool done = false;
-    if (htmlMode) {
-        done = autoFormatURLs();
+        int oldPosition = position;
+        selectPreviousWord(mCursor, position);
+        mWord = mCursor.selectedText();
+        if (mWord.isEmpty()) {
+            return true;
+        }
+        mCursor.beginEditBlock();
+        bool done = false;
+        if (htmlMode) {
+            done = autoFormatURLs();
+            if (!done) {
+                done = autoBoldUnderline();
+                //We replace */- by format => remove cursor position by 2
+                if (done) {
+                    oldPosition -= 2;
+                }
+            }
+            if (!done) {
+                superscriptAppendix();
+            }
+        }
         if (!done) {
-            done = autoBoldUnderline();
-            //We replace */- by format => remove cursor position by 2
+            done = autoFractions();
+            //We replace three characters with 1
             if (done) {
                 oldPosition -= 2;
             }
         }
         if (!done) {
-            superscriptAppendix();
+            uppercaseFirstCharOfSentence();
+            fixTwoUppercaseChars();
+            capitalizeWeekDays();
+            replaceTypographicQuotes();
+            addNonBreakingSpace();
         }
-    }
-    if (!done) {
-        done = autoFractions();
-        //We replace three characters with 1
-        if (done) {
-            oldPosition -= 2;
-        }
-    }
-    if (!done) {
-        const int newPos = advancedAutocorrect();
-        if (newPos != -1) {
-            oldPosition = newPos;
-        }
-    }
-    if (!done) {
-        uppercaseFirstCharOfSentence();
-    }
-    if (!done) {
-        fixTwoUppercaseChars();
-    }
-    if (!done) {
-        capitalizeWeekDays();
-    }
-    if (!done) {
-        replaceTypographicQuotes();
-    }
 
-    if (mCursor.selectedText() != mWord) {
-        mCursor.insertText(mWord);
+        if (mCursor.selectedText() != mWord) {
+            mCursor.insertText(mWord);
+        }
+        position = oldPosition;
+
+        if (!done) {
+            selectStringOnMaximumSearchString(mCursor, oldPosition);
+            mWord = mCursor.selectedText();
+            if (!mWord.isEmpty()) {
+                const int newPos = advancedAutocorrect();
+                if (newPos != -1) {
+                    if (mCursor.selectedText() != mWord) {
+                        mCursor.insertText(mWord);
+                    }
+                    position = newPos;
+                }
+            }
+        }
+        mCursor.endEditBlock();
     }
-    position = oldPosition;
-    mCursor.endEditBlock();
     return true;
 }
 
@@ -223,6 +265,15 @@ void AutoCorrection::setTwoUpperLetterExceptions(const QSet<QString> &exceptions
 
 void AutoCorrection::setAutocorrectEntries(const QHash<QString, QString> &entries)
 {
+    mMaxFindStringLenght = 0;
+    mMinFindStringLenght = 0;
+    QHashIterator<QString, QString> i(entries);
+    while (i.hasNext()) {
+        i.next();
+        const int findStringLenght(i.key().length());
+        mMaxFindStringLenght = qMax(mMaxFindStringLenght, findStringLenght);
+        mMinFindStringLenght = qMin(mMinFindStringLenght, findStringLenght);
+    }
     mAutocorrectEntries = entries;
 }
 
@@ -351,12 +402,13 @@ void AutoCorrection::superscriptAppendix()
     const QString trimmed = mWord.trimmed();
     int startPos = -1;
     int endPos = -1;
+    const int trimmedLenght(trimmed.length());
 
     QHash<QString, QString>::const_iterator i = mSuperScriptEntries.constBegin();
     while (i != mSuperScriptEntries.constEnd()) {
         if (i.key() == trimmed) {
             startPos = mCursor.selectionStart() + 1;
-            endPos = startPos - 1 + trimmed.length();
+            endPos = startPos - 1 + trimmedLenght;
             break;
         } else if (i.key() == QLatin1String("othernb")) {
             const int pos = trimmed.indexOf(i.value());
@@ -378,9 +430,9 @@ void AutoCorrection::superscriptAppendix()
                         ++constIter;
                     }
                 }
-                if (found && number.length() + i.value().length() == trimmed.length()) {
+                if (found && number.length() + i.value().length() == trimmedLenght) {
                     startPos = mCursor.selectionStart() + pos;
-                    endPos = startPos - pos + trimmed.length();
+                    endPos = startPos - pos + trimmedLenght;
                     break;
                 }
             }
@@ -399,6 +451,14 @@ void AutoCorrection::superscriptAppendix()
     }
 }
 
+void AutoCorrection::addNonBreakingSpace()
+{
+    if (mAddNonBreakingSpace && isFrenchLanguage()) {
+        if (mWord.length() == 1) {
+        }
+    }
+}
+
 bool AutoCorrection::autoBoldUnderline()
 {
     if (!mAutoBoldUnderline) {
@@ -410,11 +470,13 @@ bool AutoCorrection::autoBoldUnderline()
         return false;
     }
 
-    const bool underline = (trimmed.at(0) == QLatin1Char('_') && trimmed.at(trimmed.length() - 1) == QLatin1Char('_'));
-    const bool bold = (trimmed.at(0) == QLatin1Char('*') && trimmed.at(trimmed.length() - 1) == QLatin1Char('*'));
-    const bool strikeOut = (trimmed.at(0) == QLatin1Char('-') && trimmed.at(trimmed.length() - 1) == QLatin1Char('-'));
+    const QChar trimmedFirstChar(trimmed.at(0));
+    const QChar trimmedLastChar(trimmed.at(trimmed.length() - 1));
+    const bool underline = (trimmedFirstChar == QLatin1Char('_') && trimmedLastChar == QLatin1Char('_'));
+    const bool bold = (trimmedFirstChar == QLatin1Char('*') && trimmedLastChar == QLatin1Char('*'));
+    const bool strikeOut = (trimmedFirstChar == QLatin1Char('-') && trimmedLastChar == QLatin1Char('-'));
     if (underline || bold || strikeOut) {
-        int startPos = mCursor.selectionStart();
+        const int startPos = mCursor.selectionStart();
         const QString replacement = trimmed.mid(1, trimmed.length() - 2);
         bool foundLetterNumber = false;
 
@@ -465,7 +527,7 @@ bool AutoCorrection::autoFormatURLs()
     }
 
     const QString trimmed = mWord.trimmed();
-    int startPos = mCursor.selectionStart();
+    const int startPos = mCursor.selectionStart();
     mCursor.setPosition(startPos);
     mCursor.setPosition(startPos + trimmed.length(), QTextCursor::KeepAnchor);
 
@@ -722,8 +784,9 @@ bool AutoCorrection::autoFractions()
     const QString trimmed = mWord.trimmed();
     if (trimmed.length() > 3) {
         const QChar x = trimmed.at(3);
-        if (!(x.unicode() == '.' || x.unicode() == ',' || x.unicode() == '?' || x.unicode() == '!'
-                || x.unicode() == ':' || x.unicode() == ';')) {
+        const uchar xunicode = x.unicode();
+        if (!(xunicode == '.' || xunicode == ',' || xunicode == '?' || xunicode == '!'
+                || xunicode == ':' || xunicode == ';')) {
             return false;
         }
     } else if (trimmed.length() < 3) {
@@ -748,6 +811,9 @@ int AutoCorrection::advancedAutocorrect()
     if (!mAdvancedAutocorrect) {
         return -1;
     }
+    if (mAutocorrectEntries.isEmpty()) {
+        return -1;
+    }
 
     const int startPos = mCursor.selectionStart();
     const int length = mWord.length();
@@ -759,58 +825,71 @@ int AutoCorrection::advancedAutocorrect()
         return -1;
     }
 
+    const int actualWordLength(actualWord.length());
+    if (actualWordLength < mMinFindStringLenght) {
+        return -1;
+    }
+    if (actualWordLength > mMaxFindStringLenght) {
+        return -1;
+    }
+
     // If the last char is punctuation, drop it for now
     bool hasPunctuation = false;
-    QChar lastChar = actualWord.at(actualWord.length() - 1);
-    if (lastChar.unicode() == '.' || lastChar.unicode() == ',' || lastChar.unicode() == '?' ||
-            lastChar.unicode() == '!' || lastChar.unicode() == ':' || lastChar.unicode() == ';') {
+    const QChar lastChar = actualWord.at(actualWord.length() - 1);
+    const ushort charUnicode = lastChar.unicode();
+    if (charUnicode == '.' || charUnicode == ',' || charUnicode == '?' ||
+            charUnicode == '!' || charUnicode == ':' || charUnicode == ';') {
         hasPunctuation = true;
         actualWord.chop(1);
     }
 
     QString actualWordWithFirstUpperCase = actualWord;
     actualWordWithFirstUpperCase[0] = actualWordWithFirstUpperCase[0].toUpper();
-    if (mAutocorrectEntries.contains(actualWord) ||
-            mAutocorrectEntries.contains(actualWord.toLower()) ||
-            mAutocorrectEntries.contains(actualWordWithFirstUpperCase)) {
-        int pos = mWord.indexOf(trimmedWord);
-        QString replacement = mAutocorrectEntries.value(actualWord, QString());
-        if (replacement.isEmpty()) {
-            replacement = mAutocorrectEntries.value(actualWord.toLower());
-            if (replacement.isEmpty()) {
-                replacement = mAutocorrectEntries.value(actualWordWithFirstUpperCase);
+    QHashIterator<QString, QString> i(mAutocorrectEntries);
+    while (i.hasNext()) {
+        i.next();
+        if (i.key().length() > actualWordLength) {
+            continue;
+        }
+        if (actualWord.endsWith(i.key()) ||
+                actualWord.toLower().endsWith(i.key()) ||
+                actualWordWithFirstUpperCase.endsWith(i.key())) {
+            int pos = mWord.lastIndexOf(i.key());
+            if (pos == -1) {
+                pos = actualWord.toLower().lastIndexOf(i.key());
             }
-        }
-
-        // Keep capitalized words capitalized.
-        // (Necessary to make sure the first letters match???)
-        if (actualWord.at(0) == replacement.at(0).toLower()) {
-            if (mWord.at(0).isUpper()) {
-                replacement[0] = replacement[0].toUpper();
-            } else {
-                //Don't replace toUpper letter
-                if (replacement.at(0).isLower()) {
-                    replacement[0] = replacement[0].toLower();
-                }
+            if (pos == -1) {
+                pos = actualWordWithFirstUpperCase.lastIndexOf(i.key());
             }
+            QString replacement = i.value();
+
+            // Keep capitalized words capitalized.
+            // (Necessary to make sure the first letters match???)
+            const QChar actualWordFirstChar = actualWord.at(pos);
+            const QChar replacementFirstChar = replacement[0];
+            if (actualWordFirstChar.isUpper() && replacementFirstChar.isLower()) {
+                replacement[0] = replacementFirstChar.toUpper();
+            } else if (actualWordFirstChar.isLower() && replacementFirstChar.isUpper()) {
+                replacement[0] = replacementFirstChar.toLower();
+            }
+
+            // If a punctuation mark was on the end originally, add it back on
+            if (hasPunctuation) {
+                replacement.append(lastChar);
+            }
+
+            mWord.replace(pos, pos + trimmedWord.length(), replacement);
+
+            // We do replacement here, since the length of new word might be different from length of
+            // the old world. Length difference might affect other type of autocorrection
+            mCursor.setPosition(startPos);
+            mCursor.setPosition(startPos + length, QTextCursor::KeepAnchor);
+            mCursor.insertText(mWord);
+            mCursor.setPosition(startPos); // also restore the selection
+            const int newPosition = startPos + mWord.length();
+            mCursor.setPosition(newPosition, QTextCursor::KeepAnchor);
+            return newPosition;
         }
-
-        // If a punctuation mark was on the end originally, add it back on
-        if (hasPunctuation) {
-            replacement.append(lastChar);
-        }
-
-        mWord.replace(pos, pos + trimmedWord.length(), replacement);
-
-        // We do replacement here, since the length of new word might be different from length of
-        // the old world. Length difference might affect other type of autocorrection
-        mCursor.setPosition(startPos);
-        mCursor.setPosition(startPos + length, QTextCursor::KeepAnchor);
-        mCursor.insertText(mWord);
-        mCursor.setPosition(startPos); // also restore the selection
-        const int newPosition = startPos + mWord.length();
-        mCursor.setPosition(newPosition, QTextCursor::KeepAnchor);
-        return newPosition;
     }
     return -1;
 }
@@ -935,18 +1014,18 @@ void AutoCorrection::readAutoCorrectionXmlFile(bool forceGlobal)
     mTwoUpperLetterExceptions.clear();
     mSuperScriptEntries.clear();
 
-    QString LocalFile;
+    QString localFileName;
     //Look at local file:
     if (!forceGlobal) {
         if (!mAutoCorrectLang.isEmpty()) {
-            LocalFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("autocorrect/custom-") + mAutoCorrectLang + QLatin1String(".xml"));
+            localFileName = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("autocorrect/custom-") + mAutoCorrectLang + QLatin1String(".xml"));
         } else {
             if (!kdelang.isEmpty()) {
-                LocalFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("autocorrect/custom-") + kdelang + QLatin1String(".xml"));
+                localFileName = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("autocorrect/custom-") + kdelang + QLatin1String(".xml"));
             }
-            if (LocalFile.isEmpty() && kdelang.contains(QStringLiteral("_"))) {
+            if (localFileName.isEmpty() && kdelang.contains(QStringLiteral("_"))) {
                 kdelang.remove(QRegularExpression(QStringLiteral("_.*")));
-                LocalFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("autocorrect/custom-") + kdelang + QLatin1String(".xml"));
+                localFileName = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("autocorrect/custom-") + kdelang + QLatin1String(".xml"));
             }
         }
     }
@@ -977,7 +1056,7 @@ void AutoCorrection::readAutoCorrectionXmlFile(bool forceGlobal)
     //qCDebug(PIMCOMMON_LOG)<<" fname :"<<fname;
     //qCDebug(PIMCOMMON_LOG)<<" LocalFile:"<<LocalFile;
 
-    if (LocalFile.isEmpty()) {
+    if (localFileName.isEmpty()) {
         if (fname.isEmpty()) {
             mTypographicSingleQuotes = typographicDefaultSingleQuotes();
             mTypographicDoubleQuotes = typographicDefaultDoubleQuotes();
@@ -994,11 +1073,13 @@ void AutoCorrection::readAutoCorrectionXmlFile(bool forceGlobal)
                     mTypographicSingleQuotes = typographicDefaultSingleQuotes();
                     mTypographicDoubleQuotes = typographicDefaultDoubleQuotes();
                 }
+                mMaxFindStringLenght = import.maxFindStringLenght();
+                mMinFindStringLenght = import.minFindStringLenght();
             }
         }
     } else {
         ImportKMailAutocorrection import;
-        if (import.import(LocalFile, ImportAbstractAutocorrection::All)) {
+        if (import.import(localFileName, ImportAbstractAutocorrection::All)) {
             mUpperCaseExceptions = import.upperCaseExceptions();
             mTwoUpperLetterExceptions = import.twoUpperLetterExceptions();
             mAutocorrectEntries = import.autocorrectEntries();
@@ -1010,6 +1091,8 @@ void AutoCorrection::readAutoCorrectionXmlFile(bool forceGlobal)
         if (!fname.isEmpty() && import.import(fname, ImportAbstractAutocorrection::SuperScript)) {
             mSuperScriptEntries = import.superScriptEntries();
         }
+        mMaxFindStringLenght = import.maxFindStringLenght();
+        mMinFindStringLenght = import.minFindStringLenght();
     }
 }
 
